package com.tillekesoft.vlcj;


import anywheresoftware.b4a.BA;
import javafx.application.Platform;
import javafx.scene.layout.Pane;
import player.VideoPlayer;
import uk.co.caprica.vlcj.binding.LibVlc;
import uk.co.caprica.vlcj.discovery.NativeDiscovery;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.MediaPlayerEventAdapter;


@BA.Version(1.10F)
@BA.Author("Moster67 aka Mikael Osterhed")
//@BA.DependsOn(values = {"jna-3.5.2", "log4j-1.2.17", "platform-3.5.2", "vlcj-3.0.1"})
@BA.DependsOn(values = {"jna-4.1.0", "slf4j-api-1.7.10", "jna-platform-4.1.0", "vlcj-3.10.1"})
@BA.Events(values = {"Prepared", "Error", "Finished"})
@BA.ShortName("B4JVlcj")


public class B4JVlcj {
    private BA ba;
    private String eventName;
    //public Pane videoPane;
    public VideoPlayer player;
    private double VideoHeight;
    private double VideoWidth;

    public double getVersion() {
        return 1.1D;
    }

    public void LIBRARY_DOC() {
    }


    /**
     * This method should be the first code you run before initializing B4JVlcj.
     * Returns if VLC has been installed on the computer or not. If VLC has been
     * installed in a non-standard directory, VLC might not be found.
     */
    public boolean IsVLCInstalled() {
        boolean found = new NativeDiscovery().discover();
        if (!found) {
            //implement a way to inform user that VLC must be installed on the computer
            System.out.println("VLC was not found on your computer. Please install it first and then retry.");
            return false;
        }
        System.out.println("VLC version is: " + LibVlc.INSTANCE.libvlc_get_version());
        return true;
    }


    /**
     * Initializes B4JVlcj. You must set an event-name if you want to monitor events.
     * Important: Always check if VLC is installed before initializing using
     * the IsVLCInstalled-method.
     * Important: When exiting your app, always make sure to use the release-method!
     */
    public void Initialize(final BA ba, String EventName) {
        this.ba = ba;
        this.eventName = EventName.toLowerCase(BA.cul);
        //VLCUtil.discover();
        //new NativeDiscovery(new DefaultWindowsNativeDiscoveryStrategy()).discover();
        player = new VideoPlayer();
        player.setVolume(50);
        addVideoOutputEvents();
    }

    public boolean IsInitialized() {
        if (player == null) {
            return false;
        }
        return true;
    }

    /**
     * Returns Video Height
     */
    public double getVideoHeight() {
        VideoHeight = player.getVideoHeight();
        return VideoHeight;
    }

    /**
     * Returns Video Width
     */
    public double getVideoWidth() {
        VideoWidth = player.getVideoWidth();
        return VideoWidth;
    }

    /**
     * Plays media (can be a local file or a streaming source)
     */
    public void Play(String source) {
        playMedia(source);
    }

    private void playMedia(String source) {
        player.play(source);
    }

    /**
     * Stops playing media
     */
    public void Stop() {
        stopMedia();
    }

    private void stopMedia() {
        if (player != null) {
            player.stop();
        }
    }

    /**
     * Pauses media
     */
    public void Pause() {
        pauseMedia();
    }

    private void pauseMedia() {
        if (player != null) {
            player.pause();
        }
    }

    /**
     * Set the volume.
     * The volume is actually a percentage of full volume, setting a volume
     * over 100 may cause audible distortion.
     * Param: volumeLevel - a percentage of full volume in the range 0 to 200
     */
    public void SetVolume(int volumeLevel) {
        player.setVolume(volumeLevel);
    }

    /**
     * Get the current volume.
     * Returns volume, a percentage of full volume in the range 0 to 200
     */
    public int GetVolume() {
        return player.getMp().getVolume();
    }

    /**
     * Mutes volume
     */
    public void Mute() {
        player.mute();
    }

    /**
     * Unmutes volume
     */
    public void Unmute(int VolumeLevel) {
        player.unMute(VolumeLevel);
    }


    private VideoPlayer getPlayer() {
        return player;
    }

    /**
     * Is the media player playing?
     * returns true if the media player is playing, otherwise false
     */
    public boolean IsPlaying() {
        if (player.getMp().isPlaying()) {
            return true;
        }
        return false;
    }

    /**
     * Is the current media playable?
     * returns true if the current media is playable, otherwise false
     */
    public boolean IsPlayable() {
        if (player.getMp().isPlayable()) {
            return true;
        }
        return false;
    }

    /**
     * Is the current media seekable?
     * returns true if the current media is seekable, otherwise false
     */
    public boolean IsSeekable() {
        if (player.getMp().isSeekable()) {
            return true;
        }
        return false;
    }

    /**
     * Can the current media be paused?
     * returns true if the current media can be paused, otherwise false
     */
    public boolean CanPause() {
        if (player.getMp().canPause()) {
            return true;
        }
        return false;
    }


    /**
     * Get the length of the current media item.
     * Returns: length, in milliseconds
     */
    public long GetLength() {
        return player.getMp().getLength();
    }


    /**
     * Get the current play-back position.
     * Returns current position, expressed as a percentage (e.g. 0.15 is returned for 15% complete)
     */
    public float GetPosition() {
        return player.getMp().getPosition();
    }

    /**
     * Jump to a specific position. If the requested position
     * is less than zero, it is normalised to zero.
     * Param: position - position value, a percentage (e.g. 0.15 is 15%)
     */
    public void SetPosition(float position) {
        player.getMp().setPosition(position);
    }

    /**
     * Skip forward or backward by a period of time.
     * To skip backwards specify a negative delta.
     * Param: delta - time period, in milliseconds
     */
    public void Skip(long delta) {
        player.getMp().skip(delta);
    }

    /**
     * Skip forward or backward by a change in position.
     * To skip backwards specify a negative delta.
     * Param: delta - amount to skip
     */
    public void SkipPosition(float delta) {
        player.getMp().skipPosition(delta);
    }

    /**
     * Get the video aspect ratio.
     * Returns aspect ratio ONLY if you have set the AspectRatio yourself.
     */
    public String GetAspectRatio() {
        return player.getMp().getAspectRatio();
    }

    /**
     * Set the video aspect ratio
     * Param: aspectRatio - aspect ratio, e.g. "16:9", "4:3", "185:100" for 1:85.1 and so on
     */
    public void SetAspectRatio(String AspectRatio) {
        player.getMp().setAspectRatio(AspectRatio);
    }


    /**
     * Get the current video scale (zoom factor).
     */
    public float GetScale() {
        return player.getMp().getScale();
    }

    /**
     * Set the video scaling factor
     * Param: factor - scaling factor, or zero to scale the video the size of the container
     */
    public void SetScale(float factor) {
        player.getMp().setScale(factor);
    }

    /**
     * Get the current video play speed rate.
     * speedRate, where 1.0 is normal speed, 0.5 is half speed, 2.0 is double speed and so on
     */
    public float GetPlaySpeedRate() {
        return player.getMp().getRate();
    }

    /**
     * Set the video play speed rate.
     * Some media protocols are not able to change the speed rate.
     * speed rate - rate, where 1.0 is normal speed, 0.5 is half speed, 2.0 is double speed and so on
     * Returns: -1 on error, 0 on success
     */
    public int SetPlaySpeedRate(float speedRate) {
        return player.getMp().setRate(speedRate);
    }


    /**
     * Get the current play-back time.
     * Returns current time, expressed as a number of milliseconds
     */
    public long GetTime() {
        return player.getMp().getTime();
    }

    /**
     * Jump to a specific moment.
     * If the requested time is less than zero, it is normalised to zero
     * Param: time - time since the beginning, in milliseconds
     */
    public void SetTime(long time) {
        player.getMp().setTime(time);
    }


    private void addVideoOutputEvents() {
        player.getMp().addMediaPlayerEventListener(new MediaPlayerEventAdapter() {
            @Override
            public void videoOutput(MediaPlayer mediaPlayer, int i) {
                if (ba.subExists(eventName + "_prepared")) {
                    ba.raiseEvent(B4JVlcj.this, eventName + "_prepared", new Object[0]);
                }
            }

            @Override
            public void error(MediaPlayer mediaPlayer) {
                Platform.runLater(new Runnable() {
                    @Override
                    public void run() {
                        Stop();
                        if (ba.subExists(eventName + "_error")) {
                            ba.raiseEvent(B4JVlcj.this, eventName + "_error", new Object[0]);
                        }
                    }
                });
            }

            @Override
            public void finished(MediaPlayer mediaPlayer) {
                if (ba.subExists(eventName + "_finished")) {
                    ba.raiseEvent(B4JVlcj.this, eventName + "_finished", new Object[0]);
                }
            }
        });
    }


    /**
     * Use this on the b4j closing event. Final step to do is to release vlc.
     */
    public void release() {
        player.release();
    }


} //end of class
