package player;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

//import org.apache.log4j.Logger;

import com.sun.jna.Memory;

import javafx.application.Platform;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.image.PixelFormat;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritablePixelFormat;
import javafx.scene.input.ScrollEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import uk.co.caprica.vlcj.player.MediaPlayer;
import uk.co.caprica.vlcj.player.MediaPlayerEventAdapter;
import uk.co.caprica.vlcj.player.MediaPlayerEventListener;
import uk.co.caprica.vlcj.player.MediaPlayerFactory;
import uk.co.caprica.vlcj.player.direct.BufferFormat;
import uk.co.caprica.vlcj.player.direct.BufferFormatCallback;
import uk.co.caprica.vlcj.player.direct.DirectMediaPlayer;
import uk.co.caprica.vlcj.player.direct.RenderCallback;
import uk.co.caprica.vlcj.player.direct.format.RV32BufferFormat;


/**
 * Simple video player
 */
public class VideoPlayer extends BorderPane {

	/**
	 * Logger to log everything into file. See config file for more info
	 */
//	private static Logger logger = Logger.getLogger(VideoPlayer.class);
	/**
	 * Pixel format
	 */
	final private WritablePixelFormat<ByteBuffer> byteBgraInstance = PixelFormat.getByteBgraPreInstance();
	/**
	 * Canvas, where you'll see video
	 */
	protected RWImageView canvas;
	/**
	 * Media player component
	 */
	private DirectMediaPlayer mp;
	/**
	 * Pixel writer to write on WritableImage
	 */
	private PixelWriter pixelWriter = null;
	/**
	 * Inner pane, that will be put in BorderPane. ImageView will watch for its
	 * (inner pane) sizes and will match them, not parent BorderPane, to prevent
	 * overflow.
	 */
	private StackPane inner;
	/**
	 * Important player events (videoOutput, error etc)
	 */
	private MediaPlayerEventListener mediaPlayerEventListener;
	private Label volumeLabel;
	public Label itemCountLabel;
	public Label filesDeletedSizeLabel;
	private double videoWidth;
	private double videoHeight;

		public VideoPlayer() {

		List<String> arguments = new ArrayList<>();

		arguments.add("--no-plugins-cache");
		arguments.add("--no-snapshot-preview");
		arguments.add("--input-fast-seek");
		arguments.add("--no-video-title-show");
		arguments.add("--disable-screensaver");
		arguments.add("--network-caching");
		arguments.add("1000");
		arguments.add("--quiet");
		arguments.add("--quiet-synchro");
		arguments.add("--intf");
		arguments.add("dummy");
		arguments.add("--sout-keep");
		arguments.add("--verbose=2");

		canvas = new RWImageView(16, 16);
		pixelWriter = canvas.getPixelWriter();


		MediaPlayerFactory factory = new MediaPlayerFactory(arguments);

		mp = factory.newDirectMediaPlayer(
				new BufferFormatCallback() {
					@Override
					public BufferFormat getBufferFormat(final int width, final int height) {
//						logger.debug(String.format("New buffer format: %dx%d", width, height));

						Platform.runLater(new Runnable() {
							@Override
							public void run() {
								canvas.resize(width, height);
								videoHeight = height;
								videoWidth = width;
								pixelWriter = canvas.getPixelWriter();

							}
						});

						return new RV32BufferFormat(width, height);
					}
				},
				new RenderCallback() {
					@Override
					public void display(DirectMediaPlayer mp, Memory[] memory, final BufferFormat bufferFormat) {
						final ByteBuffer byteBuffer = memory[0].getByteBuffer(0, memory[0].size());

						Platform.runLater(new Runnable() {
							@Override
							public void run() {

							}
						});

						Platform.runLater(new Runnable() {
							@Override
							public void run() {
								pixelWriter.setPixels(
										0,
										0,
										bufferFormat.getWidth(),
										bufferFormat.getHeight(),
										byteBgraInstance,
										byteBuffer,
										bufferFormat.getPitches()[0]);
							}
						});
					}
				}
		);

		inner = new StackPane();
		inner.setMinSize(12, 8);
		this.setCenter(inner);
		this.setMinSize(80, 60);

		canvas.fitWidthProperty().bind(inner.widthProperty());
		canvas.fitHeightProperty().bind(inner.heightProperty());
		canvas.setPreserveRatio(true);

		inner.getChildren().add(canvas);
//		onScroll(); //disabled for now

	}

	/**
	 * set volume with mouse wheel
	 */
	private void onScroll() {
		setOnScroll(new EventHandler<ScrollEvent>() {

			@Override
			public void handle(ScrollEvent event) {
				int volume = (int) (getMp().getVolume() + (event.getDeltaY() * 0.3));
				if (volume < 0) {
					volume = 0;
				} else if (volume > 100) {
					volume = 100;
				}
				setVolume(volume);
				event.consume();
			}
		});
	}

	public double getVideoWidth() {
		return videoWidth;
	}

	public double getVideoHeight() {
		return videoHeight;
	}

	/**
	 * Play video
	 *
	 * @param path
	 *            video to play (can be anything)
	 */
	public void play(String path) {
//		logger.debug("Playing " + path);
		getMp().playMedia(path);
	}

	/*
	protected void updateTitle() {
		Platform.runLater(new Runnable() {
			@Override
			public void run() {
//				volumeLabel.setText("Volume: " + getMp().getVolume());
			}
		});
	}

	*/

	/**
	 * Stop play.
	 */
	public void stop() {
		if (getMp().isPlaying()) {
			getMp().stop();

		}
	}

	/**
	 * Pauses
	 */
	public void pause() {
		if (getMp().canPause()) {
			getMp().pause();
		}
	}

	/**
	 * Release media player.
	 */
	public void release() {
		stop();
		getMp().release();
	}

	/**
	 * Set player volume
	 *
	 * @param volume
	 */
	public void setVolume(int volume) {
		getMp().setVolume(volume);
	}

	/**
	 * Disable sound.
	 */
	public void mute() {
		getMp().setVolume(0);
	}

	/**
	 * Enable sound
	 */
	public void unMute(int volumeLevel) {
		getMp().setVolume(volumeLevel);
	}

	/*
	public void resetVolume() {
		logger.debug("sound value: " + getMp().getVolume());
		// -1: no sound yet, 0: sound is muted

		int sound = getMp().getVolume();

		if (sound < 0) {
			return;
		}

		if (sound <= 0) {
			unMute(100);
			return;
		}

	}
	*/

	/*
	public boolean skipBackward() {
		if (getMp().isPlaying()) {
			getMp().skip(-(getMp().getLength() / 10));
			return true;
		}
		return false;
	}

	public boolean skipForward() {
		if (getMp().isPlaying()) {
			getMp().skip(getMp().getLength() / 10);
			return true;
		}
		return false;
	}
	*/


	public DirectMediaPlayer getMp() {
		return mp;
	}


} //end of class